#!/usr/bin/env bash
# ----------------------------------------------------------------------------
# OpenEstate-ImmoTool 1.0.2
# create start menu items for this application
# Copyright (C) 2009-2019 OpenEstate.org
# ----------------------------------------------------------------------------

DESKTOP_DIR="$HOME/.local/share/applications"
IMMOTOOL_DESKTOP="$DESKTOP_DIR/openestate-immotool.desktop"
ADMINTOOL_DESKTOP="$DESKTOP_DIR/openestate-admintool.desktop"
MANUAL_DESKTOP="$DESKTOP_DIR/openestate-manual.desktop"


#
# Start execution...
#

SCRIPT="$( basename "${BASH_SOURCE[0]}" )"
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
BASE_DIR="$( cd "$( dirname "$SCRIPT_DIR" )" && pwd )"

if [[ ! -d "$DESKTOP_DIR" ]] ; then
    mkdir -p "$DESKTOP_DIR"
fi

if [[ ! -d "$DESKTOP_DIR" ]] ; then
    echo "Can't find local desktop directory!"
    exit 1
fi

if [[ -f "$IMMOTOOL_DESKTOP" ]] ; then
    echo "The menu item for OpenEstate-ImmoTool already exists."
else
    echo "Creating menu item for OpenEstate-ImmoTool..."
    echo "[Desktop Entry]" > "$IMMOTOOL_DESKTOP"
    echo "Name=OpenEstate-ImmoTool" >> "$IMMOTOOL_DESKTOP"
    echo "GenericName=Software for real estate agencies" >> "$IMMOTOOL_DESKTOP"
    echo "GenericName[de]=Software für Immobilienmakler" >> "$IMMOTOOL_DESKTOP"
    echo "Comment=Real estate software for small & medium sized agencies" >> "$IMMOTOOL_DESKTOP"
    echo "Comment[de]=Immobilien-Software für kleine bis mittelgroße Agenturen" >> "$IMMOTOOL_DESKTOP"
    echo "Type=Application" >> "$IMMOTOOL_DESKTOP"
    echo "Terminal=false" >> "$IMMOTOOL_DESKTOP"
    echo "Exec=$SCRIPT_DIR/ImmoTool.sh" >> "$IMMOTOOL_DESKTOP"
    echo "Path=$SCRIPT_DIR" >> "$IMMOTOOL_DESKTOP"
    echo "Icon=$BASE_DIR/share/icons/ImmoTool-128.png" >> "$IMMOTOOL_DESKTOP"
    echo "Categories=Java;Office;" >> "$IMMOTOOL_DESKTOP"
fi

if [[ -f "$ADMINTOOL_DESKTOP" ]] ; then
    echo "The menu item for OpenEstate-AdminTool already exists."
else
    echo "Creating menu item for OpenEstate-AdminTool..."
    echo "[Desktop Entry]" > "$ADMINTOOL_DESKTOP"
    echo "Name=OpenEstate-AdminTool" >> "$ADMINTOOL_DESKTOP"
    echo "Comment=Administrate projects of OpenEstate-ImmoTool" >> "$ADMINTOOL_DESKTOP"
    echo "Comment[de]=Projekte von OpenEstate-ImmoTool administrieren" >> "$ADMINTOOL_DESKTOP"
    echo "Type=Application" >> "$ADMINTOOL_DESKTOP"
    echo "Terminal=false" >> "$ADMINTOOL_DESKTOP"
    echo "Exec=$SCRIPT_DIR/AdminTool.sh" >> "$ADMINTOOL_DESKTOP"
    echo "Path=$SCRIPT_DIR" >> "$ADMINTOOL_DESKTOP"
    echo "Icon=$BASE_DIR/share/icons/AdminTool-128.png" >> "$ADMINTOOL_DESKTOP"
    echo "Categories=Java;Office;" >> "$ADMINTOOL_DESKTOP"
fi

if [[ -f "$MANUAL_DESKTOP" ]] ; then
    echo "The menu item for OpenEstate-Manual already exists."
else
    echo "Creating menu item for OpenEstate-Manual..."
    echo "[Desktop Entry]" > "$MANUAL_DESKTOP"
    echo "Name=OpenEstate-Manual" >> "$MANUAL_DESKTOP"
    echo "Name[de]=OpenEstate-Handbuch" >> "$MANUAL_DESKTOP"
    echo "Comment=Users manual for OpenEstate-ImmoTool" >> "$MANUAL_DESKTOP"
    echo "Comment[de]=Handbuch für OpenEstate-ImmoTool" >> "$MANUAL_DESKTOP"
    echo "Type=Application" >> "$MANUAL_DESKTOP"
    echo "Terminal=false" >> "$MANUAL_DESKTOP"
    echo "Exec=$SCRIPT_DIR/Manual.sh" >> "$MANUAL_DESKTOP"
    echo "Path=$SCRIPT_DIR" >> "$MANUAL_DESKTOP"
    echo "Icon=$BASE_DIR/share/icons/Manual-128.png" >> "$MANUAL_DESKTOP"
    echo "Categories=Java;Office;" >> "$MANUAL_DESKTOP"
fi
