-- -----------------------------------------------------
-- OpenEstate-ImmoTool
-- routines for HSQLDB
-- Copyright (C) 2009-2019 OpenEstate.org
-- -----------------------------------------------------
\.
-- -----------------------------------------------------
-- Function can_delete
-- -----------------------------------------------------
CREATE FUNCTION can_delete(
    permissions INT,
    owner_uid BIGINT,
    owner_gid BIGINT
  )
  RETURNS BOOLEAN
  SPECIFIC can_delete
  READS SQL DATA
  BEGIN ATOMIC
    -- Alle Anwender dürfen löschen
    IF (BITAND(permissions, 256) = 256) THEN
      RETURN TRUE;
    -- Gruppe des Anwenders darf löschen
    ELSEIF (BITAND(permissions, 32) = 32 AND owner_gid IN (SELECT group_id FROM view_immotool_users_groups WHERE user_login=USER())) THEN
      RETURN TRUE;
    -- Anwender darf löschen
    ELSEIF (BITAND(permissions, 4) = 4 AND owner_uid IN (SELECT user_id FROM view_immotool_users WHERE user_login=USER())) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;
  END;
.;
\.
-- -----------------------------------------------------
-- Function can_read
-- -----------------------------------------------------
CREATE FUNCTION can_read(
    permissions INT,
    owner_uid BIGINT,
    owner_gid BIGINT
  )
  RETURNS BOOLEAN
  SPECIFIC can_read
  READS SQL DATA
  BEGIN ATOMIC
    -- Alle Anwender dürfen lesen
    IF (BITAND(permissions, 64) = 64) THEN
      RETURN TRUE;
    -- Gruppe des Anwenders darf lesen
    ELSEIF (BITAND(permissions, 8) = 8 AND owner_gid IN (SELECT group_id FROM view_immotool_users_groups WHERE user_login=USER())) THEN
      RETURN TRUE;
    -- Anwender darf lesen
    ELSEIF (BITAND(permissions, 1) = 1 AND owner_uid IN (SELECT user_id FROM view_immotool_users WHERE user_login=USER())) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;
  END;
.;
\.
-- -----------------------------------------------------
-- Function can_write
-- -----------------------------------------------------
CREATE FUNCTION can_write(
    permissions INT,
    owner_uid BIGINT,
    owner_gid BIGINT
  )
  RETURNS BOOLEAN
  SPECIFIC can_write
  READS SQL DATA
  BEGIN ATOMIC
    -- Alle Anwender dürfen schreiben
    IF (BITAND(permissions, 128) = 128) THEN
      RETURN TRUE;
    -- Gruppe des Anwenders darf schreiben
    ELSEIF (BITAND(permissions, 16) = 16 AND owner_gid IN (SELECT group_id FROM view_immotool_users_groups WHERE user_login=USER())) THEN
      RETURN TRUE;
    -- Anwender darf schreiben
    ELSEIF (BITAND(permissions, 2) = 2 AND owner_uid IN (SELECT user_id FROM view_immotool_users WHERE user_login=USER())) THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;
  END;
.;
\.
-- -----------------------------------------------------
-- Function is_admin
-- -----------------------------------------------------
CREATE FUNCTION is_admin()
  RETURNS BOOLEAN
  SPECIFIC is_admin
  READS SQL DATA
  BEGIN ATOMIC
    DECLARE dba CHAR(3);

    IF USER() LIKE 'SA' THEN
      RETURN TRUE;
    END IF;

    -- auf Administrator-Rechte prüfen
    SELECT authorization_name
      INTO dba
      FROM information_schema.authorizations
      WHERE authorization_type = 'ROLE'
      AND authorization_name = 'DBA';
    IF dba IS NOT NULL THEN
      RETURN TRUE;
    ELSE
      RETURN FALSE;
    END IF;
  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_groups
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_groups( val_group_id BIGINT )
  SPECIFIC remove_immotool_groups
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Benutzer entfernen
    DELETE
      FROM immotool_groups
      WHERE group_id = val_group_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_groups_permissions
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_groups_permissions(
    val_group_id BIGINT,
    val_permission_id BIGINT
  )
  SPECIFIC remove_immotool_groups_permissions
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- spezifische Zuordnung entfernen
    IF val_group_id > 0 AND val_permission_id > 0 THEN

      DELETE FROM immotool_groups_permissions
        WHERE group_id = val_group_id
        AND permission_id = val_permission_id;

    -- alle Zuordnungen einer Gruppe entfernen
    ELSEIF val_group_id > 0 THEN

      DELETE
        FROM immotool_groups_permissions
        WHERE group_id = val_group_id;

    -- alle Zuordnungen einer Berechtigung entfernen
    ELSEIF val_permission_id > 0 THEN

      DELETE
        FROM immotool_groups_permissions
        WHERE permission_id = val_permission_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_logs
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_logs( val_log_id BIGINT )
  SPECIFIC remove_immotool_logs
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Protokolleintrag entfernen
    DELETE
      FROM immotool_logs
      WHERE log_id = val_log_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_logs_changes
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_logs_changes(
    val_log_id BIGINT,
    val_change_key VARCHAR(255)
  )
  SPECIFIC remove_immotool_logs_changes
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Alle Änderungen de Logs entfernen
	  IF val_change_key IS NULL THEN

	    DELETE
	      FROM immotool_logs_changes
	      WHERE log_id = val_log_id;

		-- Spezifische Änderung des Logs entfernen
	  ELSE

	    DELETE
	      FROM immotool_logs_changes
	      WHERE log_id = val_log_id
	      AND change_key = val_change_key;

  	END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE remove_immotool_logs_changes
  TO "IMMOTOOL";

\.
-- -----------------------------------------------------
-- Procedure remove_immotool_permissions
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_permissions( val_permission_id BIGINT )
  SPECIFIC remove_immotool_permissions
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Benutzer entfernen
    DELETE
      FROM immotool_permissions
      WHERE permission_id = val_permission_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_plugins
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_plugins( val_plugin_id BIGINT )
  SPECIFIC remove_immotool_plugins
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Plugin entfernen
    DELETE
      FROM immotool_plugins
      WHERE plugin_id = val_plugin_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_plugins_files
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_plugins_files( val_file_id BIGINT )
  SPECIFIC remove_immotool_plugins_files
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Datei entfernen
    DELETE
      FROM immotool_plugins_files
      WHERE file_id = val_file_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_plugins_relations
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_plugins_relations( val_relation_name VARCHAR(100) )
  SPECIFIC remove_immotool_plugins_relations
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Relation entfernen
    DELETE
      FROM immotool_plugins_relations
      WHERE relation_name = val_relation_name;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_plugins_settings
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_plugins_settings(
    val_plugin_id BIGINT,
    val_setting_key VARCHAR(255)
  )
  SPECIFIC remove_immotool_plugins_settings
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Alle Settings des Plugins entfernen
	  IF val_setting_key IS NULL THEN

	    DELETE
	      FROM immotool_plugins_settings
	      WHERE plugin_id = val_plugin_id;

		-- Spezifisches Setting des Plugins entfernen
	  ELSE

	    DELETE
	      FROM immotool_plugins_settings
	      WHERE plugin_id = val_plugin_id
	      AND setting_key = val_setting_key;

  	END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_users
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_users( val_user_id BIGINT )
  SPECIFIC remove_immotool_users
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Benutzer entfernen
    DELETE
      FROM immotool_users
      WHERE user_id = val_user_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_users_groups
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_users_groups(
    val_user_id BIGINT,
    val_group_id BIGINT
  )
  SPECIFIC remove_immotool_users_groups
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- spezifische Zuordnung entfernen
    IF val_user_id > 0 AND val_group_id > 0 THEN

      DELETE
        FROM immotool_users_groups
        WHERE user_id = val_user_id
        AND group_id = val_group_id;

    -- alle Zuordnungen eines Benutzers entfernen
    ELSEIF val_user_id > 0 THEN

      DELETE
        FROM immotool_users_groups
        WHERE user_id = val_user_id;

    -- alle Zuordnungen einer Gruppe entfernen
    ELSEIF val_group_id > 0 THEN

      DELETE
        FROM immotool_users_groups
        WHERE group_id = val_group_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_users_permissions
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_users_permissions(
    val_user_id BIGINT,
    val_permission_id BIGINT
  )
  SPECIFIC remove_immotool_users_permissions
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- spezifische Zuordnung entfernen
    IF val_user_id > 0 AND val_permission_id > 0 THEN

      DELETE
        FROM immotool_users_permissions
        WHERE user_id = val_user_id
        AND permission_id = val_permission_id;

    -- alle Zuordnungen eines Benutzers entfernen
    ELSEIF val_user_id > 0 THEN

      DELETE
        FROM immotool_users_permissions
        WHERE user_id = val_user_id;

    -- alle Zuordnungen einer Berechtigung entfernen
    ELSEIF val_permission_id > 0 THEN

      DELETE
        FROM immotool_users_permissions
        WHERE permission_id = val_permission_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure remove_immotool_users_settings
-- -----------------------------------------------------
CREATE PROCEDURE remove_immotool_users_settings(
    val_user_id BIGINT,
    val_plugin_id BIGINT,
    val_setting_key VARCHAR(255)
  )
  SPECIFIC remove_immotool_users_settings
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE temp_user_id BIGINT;

    IF is_admin() = FALSE THEN
      -- sicherstellen, dass der Benutzer nur seine eigenen Daten bearbeiten darf
      SELECT user_id
        INTO temp_user_id
        FROM immotool_users
        WHERE user_login = USER();
      IF NOT val_user_id = temp_user_id THEN
        SIGNAL SQLSTATE '45000';
      END IF;
    END IF;

    -- Alle Settings des Benutzers entfernen
    IF val_plugin_id < 1 THEN

	    DELETE
	      FROM immotool_users_settings
	      WHERE user_id = val_user_id;

		-- Settings des Benutzers für ein Plugin entfernen
	  ELSEIF val_setting_key IS NULL THEN

	    DELETE
	      FROM immotool_users_settings
	      WHERE user_id = val_user_id
	      AND plugin_id = val_plugin_id;

		-- Spezifisches Setting des Benutzers entfernen
	  ELSE

	    DELETE
	      FROM immotool_users_settings
	      WHERE user_id = val_user_id
	      AND plugin_id = val_plugin_id
	      AND setting_key = val_setting_key;

  	END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE remove_immotool_users_settings
  TO "IMMOTOOL";

\.
-- -----------------------------------------------------
-- Procedure save_immotool_groups
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_groups(
    INOUT val_group_id BIGINT,
    val_group_name VARCHAR(100),
    val_group_notes LONGVARCHAR
  )
  SPECIFIC save_immotool_groups
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Neue Gruppe
    IF val_group_id < 1 THEN

      -- Gruppe speichern
      INSERT INTO immotool_groups
        (group_name, group_notes)
      VALUES
        (val_group_name, val_group_notes);

      -- ID zurückliefern
      SET val_group_id = IDENTITY();

    -- Bestehende Gruppe
    ELSE

      -- Gruppe bearbeiten
      UPDATE immotool_groups
        SET
          group_name = val_group_name,
          group_notes = val_group_notes,
          modified_at = NOW()
        WHERE
          group_id = val_group_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_groups_permissions
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_groups_permissions(
    val_group_id BIGINT,
    val_permission_id BIGINT
  )
  SPECIFIC save_immotool_groups_permissions
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Existierenden Eintrag entfernen, wenn vorhanden
    DELETE
      FROM immotool_groups_permissions
      WHERE group_id = val_group_id
      AND permission_id = val_permission_id;

    -- Eintrag speichern, wenn ein Wert angegeben wurde
    IF (val_group_id > 0 AND val_permission_id > 0) THEN

      INSERT INTO immotool_groups_permissions
        (group_id, permission_id)
      VALUES
        (val_group_id, val_permission_id);

  	END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_logs
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_logs(
    INOUT val_log_id BIGINT,
    val_plugin_name VARCHAR(255),
    val_log_entry_type VARCHAR(255),
    val_log_entry_action VARCHAR(255),
    val_log_text LONGVARCHAR,
    val_log_notes LONGVARCHAR,
    val_created_by VARCHAR(255),
    val_modified_by VARCHAR(255)
  )
  SPECIFIC save_immotool_logs
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE val_plugin_id BIGINT;

    -- ID des zugewiesenen Plugins ermitteln
    SELECT plugin_id
      INTO val_plugin_id
      FROM immotool_plugins
      WHERE plugin_name = val_plugin_name;

    -- Neuer Protokolleintrag
    IF val_log_id < 1 THEN

      -- Protokolleintrag speichern
      INSERT INTO immotool_logs (
        plugin_id,
        log_entry_type,
        log_entry_action,
        log_text,
        log_notes,
        created_by,
        modified_by )
      VALUES (
        val_plugin_id,
        val_log_entry_type,
        val_log_entry_action,
        val_log_text,
        val_log_notes,
        val_created_by,
        val_modified_by );

      -- ID zurückliefern
      SET val_log_id = IDENTITY();

    -- Bestehender Protokolleintrag
    ELSE

      -- Protokolleintrag bearbeiten
      UPDATE immotool_logs
        SET
          plugin_id = val_plugin_id,
          log_entry_type = val_log_entry_type,
          log_entry_action = val_log_entry_action,
          log_text = val_log_text,
          log_notes = val_log_notes,
          created_by = val_created_by,
          modified_by = val_modified_by,
          modified_at = NOW()
        WHERE
          log_id = val_log_id;

    END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_logs
  TO "IMMOTOOL";

\.
-- -----------------------------------------------------
-- Procedure save_immotool_logs_changes
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_logs_changes(
    val_log_id BIGINT,
    val_change_key VARCHAR(255),
    val_change_value_old LONGVARCHAR,
    val_change_value_new LONGVARCHAR
  )
  SPECIFIC save_immotool_logs_changes
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Existierende Änderung entfernen, wenn vorhanden
    DELETE
      FROM immotool_logs_changes
      WHERE log_id = val_log_id
      AND change_key = val_change_key;

    -- Änderung speichern
    INSERT INTO immotool_logs_changes (
      log_id,
      change_key,
      change_value_old,
      change_value_new )
    VALUES (
      val_log_id,
      val_change_key,
      val_change_value_old,
      val_change_value_new );

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_logs_changes
  TO "IMMOTOOL";

\.
-- -----------------------------------------------------
-- Procedure save_immotool_permissions
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_permissions(
    INOUT val_permission_id BIGINT,
    val_plugin_id BIGINT,
    val_permission_key VARCHAR(100)
  )
  SPECIFIC save_immotool_permissions
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Neue Berechtigung
    IF val_permission_id < 1 THEN

      -- Berechtigung speichern
      INSERT INTO immotool_permissions
        (plugin_id, permission_key)
      VALUES
        (val_plugin_id, val_permission_key);

      -- ID zurückliefern
      SET val_permission_id = IDENTITY();

    -- Bestehende Berechtigung
    ELSE

      -- Berechtigung bearbeiten
      UPDATE immotool_permissions
        SET
          plugin_id = val_plugin_id,
          permission_key = val_permission_key,
          modified_at = NOW()
        WHERE
          permission_id = val_permission_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_plugins
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_plugins(
    INOUT val_plugin_id BIGINT,
    val_plugin_name VARCHAR(100),
    val_plugin_version VARCHAR(25),
    val_plugin_api BIGINT,
    val_plugin_enabled BOOLEAN
  )
  SPECIFIC save_immotool_plugins
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Neues Plugin
    IF val_plugin_id < 1 THEN

      -- Plugin speichern
      INSERT INTO immotool_plugins (
        plugin_name,
        plugin_version,
        plugin_api,
        plugin_enabled )
      VALUES (
        val_plugin_name,
        val_plugin_version,
        val_plugin_api,
        val_plugin_enabled );

      -- ID zurückliefern
      SET val_plugin_id = IDENTITY();

    -- Bestehendes Plugin
    ELSE

      -- Plugin bearbeiten
      UPDATE immotool_plugins
        SET
          plugin_name = val_plugin_name,
          plugin_version = val_plugin_version,
          plugin_api = val_plugin_api,
          plugin_enabled = val_plugin_enabled,
          modified_at = NOW()
        WHERE
          plugin_id = val_plugin_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_plugins_files
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_plugins_files(
    INOUT val_file_id BIGINT,
    val_plugin_id BIGINT,
    val_file_name VARCHAR(255),
    val_file_mimetype VARCHAR(255),
    val_file_content BLOB(1G)
  )
  SPECIFIC save_immotool_plugins_files
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Neue Datei
    IF val_file_id < 1 THEN

      -- Datei speichern
      INSERT INTO immotool_plugins_files (
        plugin_id,
        file_name,
        file_mimetype )
      VALUES (
        val_plugin_id,
        val_file_name,
        val_file_mimetype );

      -- ID zurückliefern
      SET val_file_id = IDENTITY();

    -- Bestehende Datei
    ELSE

      -- Datei bearbeiten
      UPDATE immotool_plugins_files
        SET
          plugin_id = val_plugin_id,
          file_name = val_file_name,
          file_mimetype = val_file_mimetype,
          modified_at = NOW()
        WHERE
          file_id = val_file_id;

    END IF;

    -- Datei ggf. aktualisieren
    IF val_file_content IS NOT NULL THEN

      -- Datei bearbeiten
      UPDATE immotool_plugins_files
        SET
          file_content = val_file_content,
          modified_at = NOW()
        WHERE
          file_id = val_file_id;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_plugins_relations
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_plugins_relations(
    val_relation_name VARCHAR(100),
    val_relation_1st_plugin_id BIGINT,
    val_relation_2nd_plugin_id BIGINT,
    val_relation_api BIGINT
  )
  SPECIFIC save_immotool_plugins_relations
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE temp_name VARCHAR(100);

    -- prüfen, ob bereits ein Eintrag existiert
    SELECT val_relation_name
      INTO temp_name
      FROM immotool_plugins_relations
      WHERE relation_name = val_relation_name;

    -- neuen Eintrag speichern
    IF temp_name IS NULL THEN
      INSERT INTO immotool_plugins_relations (
        relation_name,
        relation_1st_plugin_id,
        relation_2nd_plugin_id,
        relation_api )
      VALUES (
        val_relation_name,
        val_relation_1st_plugin_id,
        val_relation_2nd_plugin_id,
        val_relation_api );

    -- bestehenden Eintrag aktualisieren
    ELSE
      UPDATE immotool_plugins_relations
        SET
          relation_1st_plugin_id = val_relation_1st_plugin_id,
          relation_2nd_plugin_id = val_relation_2nd_plugin_id,
          relation_api = val_relation_api,
          modified_at = NOW()
        WHERE
          relation_name = val_relation_name;

    END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_plugins_settings
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_plugins_settings(
    val_plugin_id BIGINT,
    val_setting_key VARCHAR(255),
    val_setting_value LONGVARCHAR
  )
  SPECIFIC save_immotool_plugins_settings
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Existierenden Eintrag entfernen, wenn vorhanden
    DELETE
      FROM immotool_plugins_settings
      WHERE plugin_id = val_plugin_id
      AND setting_key = val_setting_key;

    -- Eintrag speichern, wenn ein Wert angegeben wurde
    IF (val_setting_value IS NOT NULL) THEN

      INSERT INTO immotool_plugins_settings (
        plugin_id,
        setting_key,
        setting_value )
      VALUES (
        val_plugin_id,
        val_setting_key,
        val_setting_value );

  	END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_users
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_users(
    INOUT val_user_id BIGINT,
    val_user_login VARCHAR(100),
    val_user_notes LONGVARCHAR
  )
  SPECIFIC save_immotool_users
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE temp_user_id BIGINT;
    DECLARE new_user_login VARCHAR(100);

    -- Neuer Benutzer
    IF val_user_id < 1 THEN

      -- sicherstellen, dass der Benutzer ein Administrator ist
      IF is_admin() = FALSE THEN
        SIGNAL SQLSTATE '45000';
      END IF;

      -- Benutzer speichern
      INSERT INTO immotool_users (
        user_login,
        user_notes )
      VALUES (
        val_user_login,
        val_user_notes );

      -- ID zurückliefern
      SET val_user_id = IDENTITY();

    -- Bestehender Benutzer
    ELSE

      IF is_admin() = TRUE THEN
        -- Login-Name darf nur von Administratoren geändert werden
        SET new_user_login = val_user_login;

      ELSE
        -- sicherstellen, dass der Benutzer nur seine eigenen Daten bearbeiten darf
        SELECT user_id, user_login
          INTO temp_user_id, new_user_login
          FROM immotool_users
          WHERE user_login = USER();
        IF NOT val_user_id = temp_user_id THEN
          SIGNAL SQLSTATE '45000';
        END IF;
      END IF;

      -- Benutzer bearbeiten
      UPDATE immotool_users
        SET
          user_login = new_user_login,
          user_notes = val_user_notes,
          modified_at = NOW()
        WHERE
          user_id = val_user_id;

    END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_users
  TO "IMMOTOOL";

\.
-- -----------------------------------------------------
-- Procedure save_immotool_users_groups
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_users_groups(
    val_user_id BIGINT,
    val_group_id BIGINT
  )
  SPECIFIC save_immotool_users_groups
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Existierenden Eintrag entfernen, wenn vorhanden
    DELETE
      FROM immotool_users_groups
      WHERE user_id = val_user_id
      AND group_id = val_group_id;

    -- Eintrag speichern, wenn ein Wert angegeben wurde
    IF (val_user_id > 0 AND val_group_id > 0) THEN

      INSERT INTO immotool_users_groups (
        user_id,
        group_id )
      VALUES (
        val_user_id,
        val_group_id );

  	END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_users_permissions
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_users_permissions(
    val_user_id BIGINT,
    val_permission_id BIGINT
  )
  SPECIFIC save_immotool_users_permissions
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Existierenden Eintrag entfernen, wenn vorhanden
    DELETE
      FROM immotool_users_permissions
      WHERE user_id = val_user_id
      AND permission_id = val_permission_id;

    -- Eintrag speichern, wenn ein Wert angegeben wurde
    IF (val_user_id > 0 AND val_permission_id > 0) THEN

      INSERT INTO immotool_users_permissions (
        user_id,
        permission_id )
      VALUES (
        val_user_id,
        val_permission_id );

  	END IF;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure save_immotool_users_settings
-- -----------------------------------------------------
CREATE PROCEDURE save_immotool_users_settings(
    val_user_id BIGINT,
    val_plugin_id BIGINT,
    val_setting_key VARCHAR(255),
    val_setting_value LONGVARCHAR
  )
  SPECIFIC save_immotool_users_settings
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE temp_user_id BIGINT;

    IF is_admin() = FALSE THEN
      -- sicherstellen, dass der Benutzer nur seine eigenen Daten bearbeiten darf
      SELECT user_id
        INTO temp_user_id
        FROM immotool_users
        WHERE user_login = USER();
      IF NOT val_user_id = temp_user_id THEN
        SIGNAL SQLSTATE '45000';
      END IF;
    END IF;

    -- Existierenden Eintrag entfernen, wenn vorhanden
    DELETE
      FROM immotool_users_settings
      WHERE user_id = val_user_id
      AND plugin_id = val_plugin_id
      AND setting_key = val_setting_key;

    -- Eintrag speichern, wenn ein Wert angegeben wurde
    IF (val_setting_value IS NOT NULL) THEN

      INSERT INTO immotool_users_settings (
        user_id,
        plugin_id,
        setting_key,
        setting_value )
      VALUES (
        val_user_id,
        val_plugin_id,
        val_setting_key,
        val_setting_value );

  	END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE save_immotool_users_settings
  TO "IMMOTOOL";

\.
-- -----------------------------------------------------
-- Procedure set_immotool_plugins_enabled
-- -----------------------------------------------------
CREATE PROCEDURE set_immotool_plugins_enabled(
    val_plugin_id BIGINT,
    val_plugin_enabled BOOLEAN
  )
  SPECIFIC set_immotool_plugins_enabled
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Plugin bearbeiten
    UPDATE immotool_plugins
      SET
        plugin_enabled = val_plugin_enabled,
        modified_at = NOW()
      WHERE
        plugin_id = val_plugin_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure set_immotool_plugins_relation_uninstall
-- -----------------------------------------------------
CREATE PROCEDURE set_immotool_plugins_relation_uninstall(
    val_relation_name VARCHAR(100),
    val_relation_uninstall LONGVARCHAR
  )
  SPECIFIC set_immotool_plugins_relation_uninstall
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Relation bearbeiten
    UPDATE immotool_plugins_relations
      SET
        relation_uninstall = val_relation_uninstall,
        modified_at = NOW()
      WHERE
        relation_name = val_relation_name;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure set_immotool_plugins_uninstall
-- -----------------------------------------------------
CREATE PROCEDURE set_immotool_plugins_uninstall(
    val_plugin_id BIGINT,
    val_plugin_uninstall LONGVARCHAR
  )
  SPECIFIC set_immotool_plugins_uninstall
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- sicherstellen, dass der Benutzer ein Administrator ist
    IF is_admin() = FALSE THEN
      SIGNAL SQLSTATE '45000';
    END IF;

    -- Plugin bearbeiten
    UPDATE immotool_plugins
      SET
        plugin_uninstall = val_plugin_uninstall,
        modified_at = NOW()
      WHERE
        plugin_id = val_plugin_id;

  END;
.;
\.
-- -----------------------------------------------------
-- Procedure set_immotool_users_last_login
-- -----------------------------------------------------
CREATE PROCEDURE set_immotool_users_last_login(
    val_user_id BIGINT,
    val_last_login_at TIMESTAMP WITH TIME ZONE
  )
  SPECIFIC set_immotool_users_last_login
  MODIFIES SQL DATA
  BEGIN ATOMIC
    DECLARE temp_user_id BIGINT;

    IF is_admin() = FALSE THEN
      -- sicherstellen, dass der Benutzer nur seine eigenen Daten bearbeiten darf
      SELECT user_id
        INTO temp_user_id
        FROM immotool_users
        WHERE user_login = USER();
      IF NOT val_user_id = temp_user_id THEN
        SIGNAL SQLSTATE '45000';
      END IF;
    END IF;

    -- Zeitpunkt der letzten Anmeldung übernehmen
    IF val_last_login_at IS NULL THEN
      UPDATE immotool_users
        SET
          last_login_at = NOW()
        WHERE
          user_id = val_user_id;
    ELSE
      UPDATE immotool_users
        SET
          last_login_at = val_last_login_at
        WHERE
          user_id = val_user_id;
    END IF;

  END;
.;

GRANT EXECUTE
  ON PROCEDURE set_immotool_users_last_login
  TO "IMMOTOOL";
