-- -----------------------------------------------------
-- OpenEstate-ImmoTool
-- schema for HSQLDB
-- Copyright (C) 2009-2019 OpenEstate.org
-- -----------------------------------------------------

SET TABLE SYSTEM_LOBS.BLOCKS TYPE CACHED
SET TABLE SYSTEM_LOBS.LOBS TYPE CACHED
SET TABLE SYSTEM_LOBS.LOB_IDS TYPE CACHED
SET TABLE SYSTEM_LOBS.PARTS TYPE CACHED

CREATE ROLE "IMMOTOOL";
CREATE SEQUENCE seq_immotool_logs
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;
CREATE SEQUENCE seq_immotool_plugins
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;
CREATE SEQUENCE seq_immotool_plugins_files
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;
CREATE SEQUENCE seq_immotool_users
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;
CREATE SEQUENCE seq_immotool_groups
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;
CREATE SEQUENCE seq_immotool_permissions
  AS BIGINT START WITH 1 INCREMENT BY 1 NO CYCLE;


-- -----------------------------------------------------
-- Table immotool_plugins
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_plugins (
  plugin_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_plugins NOT NULL,
  plugin_name VARCHAR(100) NOT NULL,
  plugin_version VARCHAR(25) NOT NULL,
  plugin_api BIGINT DEFAULT 0 NOT NULL,
  plugin_enabled BOOLEAN DEFAULT true NOT NULL,
  plugin_uninstall LONGVARCHAR DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  PRIMARY KEY(plugin_id),
  UNIQUE(plugin_name)
);


-- -----------------------------------------------------
-- Table immotool_plugins_files
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_plugins_files (
  file_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_plugins_files NOT NULL,
  plugin_id BIGINT NOT NULL,
  file_name VARCHAR(255) NOT NULL,
  file_mimetype VARCHAR(255) NULL,
  file_content BLOB(1G) DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  PRIMARY KEY (file_id),
  UNIQUE (plugin_id, file_name),
  FOREIGN KEY (plugin_id)
    REFERENCES immotool_plugins (plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_plugins_relations
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_plugins_relations (
  relation_name VARCHAR(255) NOT NULL,
  relation_1st_plugin_id BIGINT NOT NULL,
  relation_2nd_plugin_id BIGINT NOT NULL,
  relation_api BIGINT DEFAULT 0 NOT NULL,
  relation_uninstall LONGVARCHAR DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  PRIMARY KEY (relation_name),
  FOREIGN KEY (relation_1st_plugin_id)
    REFERENCES immotool_plugins(plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY (relation_2nd_plugin_id)
    REFERENCES immotool_plugins(plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_plugins_settings
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_plugins_settings (
  plugin_id BIGINT NOT NULL,
  setting_key VARCHAR(255) NOT NULL,
  setting_value LONGVARCHAR DEFAULT NULL NULL,
  PRIMARY KEY (plugin_id, setting_key),
  FOREIGN KEY (plugin_id)
    REFERENCES immotool_plugins(plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_users
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_users (
  user_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_users NOT NULL,
  user_login VARCHAR(100) NOT NULL,
  user_notes LONGVARCHAR DEFAULT NULL NULL,
  last_login_at TIMESTAMP WITH TIME ZONE DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  PRIMARY KEY (user_id),
  UNIQUE(user_login)
);


-- -----------------------------------------------------
-- Table immotool_groups
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_groups (
  group_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_groups NOT NULL,
  group_name VARCHAR(100) NOT NULL,
  group_notes LONGVARCHAR DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  PRIMARY KEY (group_id),
  UNIQUE(group_name)
);


-- -----------------------------------------------------
-- Table immotool_users_groups
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_users_groups (
  user_id BIGINT NOT NULL,
  group_id BIGINT NOT NULL,
  PRIMARY KEY (user_id, group_id),
  FOREIGN KEY (user_id)
    REFERENCES immotool_users (user_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY (group_id)
    REFERENCES immotool_groups (group_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_permissions
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_permissions (
  permission_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_permissions NOT NULL,
  plugin_id BIGINT NOT NULL,
  permission_key VARCHAR(255) NOT NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  PRIMARY KEY (permission_id),
  UNIQUE(plugin_id, permission_key),
  FOREIGN KEY (plugin_id)
    REFERENCES immotool_plugins (plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_groups_permissions
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_groups_permissions (
  group_id BIGINT NOT NULL,
  permission_id BIGINT NOT NULL,
  PRIMARY KEY (group_id, permission_id),
  FOREIGN KEY (group_id)
    REFERENCES immotool_groups (group_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY (permission_id)
    REFERENCES immotool_permissions (permission_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_users_permissions
-- -----------------------------------------------------
CREATE MEMORY TABLE immotool_users_permissions (
  user_id BIGINT NOT NULL,
  permission_id BIGINT NOT NULL,
  PRIMARY KEY (user_id, permission_id),
  FOREIGN KEY (user_id)
    REFERENCES immotool_users (user_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY (permission_id)
    REFERENCES immotool_permissions (permission_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_users_settings
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_users_settings (
  user_id BIGINT NOT NULL,
  plugin_id BIGINT NOT NULL,
  setting_key VARCHAR(255) NOT NULL,
  setting_value LONGVARCHAR DEFAULT NULL NULL,
  PRIMARY KEY (user_id, plugin_id, setting_key),
  FOREIGN KEY (user_id)
    REFERENCES immotool_users (user_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY (plugin_id)
    REFERENCES immotool_plugins (plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_logs
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_logs (
  log_id BIGINT GENERATED BY DEFAULT AS SEQUENCE seq_immotool_logs NOT NULL,
  plugin_id BIGINT NOT NULL,
  log_entry_type VARCHAR(255) NOT NULL,
  log_entry_action VARCHAR(255) DEFAULT NULL NULL,
  log_text LONGVARCHAR DEFAULT NULL NULL,
  log_notes LONGVARCHAR DEFAULT NULL NULL,
  created_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  created_by VARCHAR(255) DEFAULT NULL NULL,
  modified_at TIMESTAMP WITH TIME ZONE DEFAULT NOW() NOT NULL,
  modified_by VARCHAR(255) DEFAULT NULL NULL,
  PRIMARY KEY(log_id),
  FOREIGN KEY (plugin_id)
    REFERENCES immotool_plugins (plugin_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- Table immotool_logs_changes
-- -----------------------------------------------------
CREATE CACHED TABLE immotool_logs_changes (
  log_id BIGINT NOT NULL,
  change_key VARCHAR(255) NOT NULL,
  change_value_old LONGVARCHAR DEFAULT NULL NULL,
  change_value_new LONGVARCHAR DEFAULT NULL NULL,
  PRIMARY KEY (log_id, change_key),
  FOREIGN KEY (log_id)
    REFERENCES immotool_logs(log_id)
    MATCH FULL
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- -----------------------------------------------------
-- View view_immotool_users
-- -----------------------------------------------------
CREATE VIEW view_immotool_users AS
  SELECT * FROM immotool_users;

GRANT SELECT
  ON view_immotool_users
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_users_permissions
-- -----------------------------------------------------
CREATE VIEW view_immotool_users_permissions AS
  SELECT
    users.user_id,
    users.user_login,
    users.user_notes,
    plugins.plugin_id,
    plugins.plugin_name,
    perms.permission_id,
    perms.permission_key
  FROM immotool_users_permissions AS relation
    INNER JOIN immotool_users AS users ON (relation.user_id=users.user_id)
    INNER JOIN immotool_permissions AS perms ON (relation.permission_id=perms.permission_id)
    INNER JOIN immotool_plugins AS plugins ON (perms.plugin_id = plugins.plugin_id);

GRANT SELECT
  ON view_immotool_users_permissions
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_users_settings
-- -----------------------------------------------------
CREATE VIEW view_immotool_users_settings AS
  SELECT
    users.user_id,
    plugins.plugin_id,
    plugins.plugin_name,
    settings.setting_key,
    settings.setting_value
  FROM immotool_users_settings AS settings
    INNER JOIN immotool_users AS users ON ( settings.user_id = users.user_id )
    INNER JOIN immotool_plugins AS plugins ON ( settings.plugin_id = plugins.plugin_id );

GRANT SELECT
  ON view_immotool_users_settings
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_plugins
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins AS
  SELECT
    plugin_id,
    plugin_name,
    plugin_version,
    plugin_api,
    plugin_enabled,
    created_at,
    modified_at
  FROM immotool_plugins;

GRANT SELECT
  ON view_immotool_plugins
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_plugins_files
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins_files AS
  SELECT
    file_id,
    plugin_id,
    file_name,
    file_mimetype,
    created_at,
    modified_at
  FROM immotool_plugins_files;

GRANT SELECT
  ON view_immotool_plugins_files
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_plugins_files_content
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins_files_content AS
  SELECT
    file_id,
    file_content,
    created_at,
    modified_at
  FROM immotool_plugins_files;

GRANT SELECT
  ON view_immotool_plugins_files_content
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_plugins_relations
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins_relations AS
  SELECT
    relation_name,
    relation_1st_plugin_id,
    relation_2nd_plugin_id,
    relation_api,
    created_at,
    modified_at
  FROM immotool_plugins_relations;

GRANT SELECT
  ON view_immotool_plugins_relations
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_plugins_relations_uninstall
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins_relations_uninstall AS
  SELECT
    relation_name,
    relation_1st_plugin_id,
    relation_2nd_plugin_id,
    relation_uninstall
  FROM immotool_plugins_relations;


-- -----------------------------------------------------
-- View view_immotool_plugins_settings
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins_settings AS
  SELECT * FROM immotool_plugins_settings;

GRANT SELECT
  ON view_immotool_plugins_settings
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_plugins_uninstall
-- -----------------------------------------------------
CREATE VIEW view_immotool_plugins_uninstall AS
  SELECT
    plugin_id,
    plugin_name,
    plugin_uninstall
  FROM immotool_plugins;


-- -----------------------------------------------------
-- View view_immotool_groups
-- -----------------------------------------------------
CREATE VIEW view_immotool_groups AS
  SELECT * FROM immotool_groups;

GRANT SELECT
  ON view_immotool_groups
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_users_groups
-- -----------------------------------------------------
CREATE VIEW view_immotool_users_groups AS
  SELECT
    u.user_id,
    u.user_login,
    g.group_id,
    g.group_name
  FROM immotool_users_groups AS groups
    INNER JOIN immotool_users AS u ON (groups.user_id=u.user_id)
    INNER JOIN immotool_groups AS g ON (groups.group_id=g.group_id);

GRANT SELECT
  ON view_immotool_users_groups
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_permissions
-- -----------------------------------------------------
CREATE VIEW view_immotool_permissions AS
  SELECT * FROM immotool_permissions;

GRANT SELECT
  ON view_immotool_permissions
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_groups_permissions
-- -----------------------------------------------------
CREATE VIEW view_immotool_groups_permissions AS
  SELECT
    g.group_id,
    g.group_name,
    g.group_notes,
    plug.plugin_id,
    plug.plugin_name,
    perm.permission_id,
    perm.permission_key
  FROM immotool_groups_permissions AS relation
    INNER JOIN immotool_groups AS g ON (relation.group_id=g.group_id)
    INNER JOIN immotool_permissions AS perm ON (relation.permission_id=perm.permission_id)
    INNER JOIN immotool_plugins AS plug ON (perm.plugin_id=plug.plugin_id);

GRANT SELECT
  ON view_immotool_groups_permissions
  TO "IMMOTOOL";


-- -----------------------------------------------------
-- View view_immotool_logs_changes
-- -----------------------------------------------------
CREATE VIEW view_immotool_logs_changes AS
  SELECT * FROM immotool_logs_changes;

GRANT SELECT
  ON view_immotool_logs_changes
  TO "IMMOTOOL";
