-- -----------------------------------------------------
-- OpenEstate-ImmoTool
-- update 4 for HSQLDB
-- Copyright (C) 2009-2019 OpenEstate.org
-- -----------------------------------------------------
-- Set type of 'file_mimetype' in 'immotool_plugins_files' to 'VARCHAR(255)'
-- Update 'save_immotool_plugins_files'
-- Update 'view_immotool_plugins_files'
-- -----------------------------------------------------

-- -----------------------------------------------------
-- Set type of 'file_mimetype' in 'immotool_plugins_files' to 'VARCHAR(255)'
-- -----------------------------------------------------
ALTER TABLE immotool_plugins_files
  ALTER COLUMN file_mimetype SET DATA TYPE VARCHAR(255);

-- -----------------------------------------------------
-- Update 'save_immotool_plugins_files'
-- -----------------------------------------------------
DROP PROCEDURE IF EXISTS save_immotool_plugins_files RESTRICT;
\.
CREATE PROCEDURE save_immotool_plugins_files(
    INOUT val_file_id BIGINT,
    val_plugin_id BIGINT,
    val_file_name VARCHAR(255),
    val_file_mimetype VARCHAR(255),
    val_file_content BLOB(1G)
  )
  SPECIFIC save_immotool_plugins_files
  MODIFIES SQL DATA
  BEGIN ATOMIC

    -- Neue Datei
    IF val_file_id < 1 THEN

      -- Datei speichern
      INSERT INTO immotool_plugins_files (
        plugin_id,
        file_name,
        file_mimetype )
      VALUES (
        val_plugin_id,
        val_file_name,
        val_file_mimetype );

      -- ID zurückliefern
      SET val_file_id = IDENTITY();

    -- Bestehende Datei
    ELSE

      -- Datei bearbeiten
      UPDATE immotool_plugins_files
        SET
          plugin_id = val_plugin_id,
          file_name = val_file_name,
          file_mimetype = val_file_mimetype,
          modified_at = NOW()
        WHERE
          file_id = val_file_id;

    END IF;

    -- Datei ggf. aktualisieren
    IF val_file_content IS NOT NULL THEN

      -- Datei bearbeiten
      UPDATE immotool_plugins_files
        SET
          file_content = val_file_content,
          modified_at = NOW()
        WHERE
          file_id = val_file_id;

    END IF;

  END;
.;
GRANT EXECUTE
  ON PROCEDURE save_immotool_plugins_files
  TO "IMMOTOOL";

-- -----------------------------------------------------
-- Update 'view_immotool_plugins_files'
-- -----------------------------------------------------
ALTER VIEW view_immotool_plugins_files AS
  SELECT
    file_id,
    plugin_id,
    file_name,
    file_mimetype,
    created_at,
    modified_at
  FROM immotool_plugins_files;
